part of model_notifier;

extension ChangeNotifierMapExtensions<T extends ChangeNotifier> on Iterable<T> {
  /// Convert the list of [ChangeNotifier] to the list of widgets.
  ///
  /// It not only converts,
  /// but also listens for changes in each item and rebuilds only that part of the item if there are changes in each item.
  ///
  /// Easily create high performance lists.
  ///
  /// [callback] with the content of the widget.
  List<Widget> mapWidget(Widget? Function(T item) callback) {
    return map((item) {
      return ChangeNotifierListener<T>(
        notifier: item,
        builder: (context, notifier) =>
            callback.call(notifier) ?? const SizedBox(),
      );
    }).toList();
  }
}

extension NullableAsyncValueExtensions<T> on AsyncValue<T>? {
  /// Get the value.
  ///
  /// If it is itself null or the value is null, [defaultValue] is returned.
  T value(T defaultValue) {
    if (this == null) {
      return defaultValue;
    }
    return this!.data?.value ?? defaultValue;
  }
}

extension AsyncValueExtensions<T> on AsyncValue<T> {
  /// Get the value.
  ///
  /// If it is itself null or the value is null, [defaultValue] is returned.
  T value(T defaultValue) {
    return data?.value ?? defaultValue;
  }
}

extension _RuntimeStringMapExtensions on DynamicMap {
  T? _readFromPath<T>(String path) {
    final paths = path.split("/");
    if (paths.isEmpty) {
      return null;
    }
    return _readFromPathInternal(paths, 0) as T;
  }

  dynamic _readFromPathInternal(List<String> paths, int index) {
    if (paths.length <= index) {
      return this;
    }
    final p = paths[index];
    if (p.isEmpty || !containsKey(p)) {
      return null;
    }
    if (this[p] is DynamicMap)
      return (this[p] as DynamicMap)._readFromPathInternal(paths, index + 1);
    return this[p];
  }

  void _writeToPath(String path, dynamic value) {
    final paths = path.split("/");
    if (paths.isEmpty) {
      return;
    }
    _writeToPathInternal(paths, 0, value);
  }

  void _writeToPathInternal(List<String> paths, int index, dynamic value) {
    if (paths.length - 1 <= index) {
      remove(paths[index]);
      this[paths[index]] = value;
      return;
    }
    final p = paths[index];
    if (p.isEmpty) {
      return;
    }
    if (!containsKey(p) || this[p] == null || this[p] is! DynamicMap) {
      this[p] = <String, dynamic>{};
    }
    (this[p] as DynamicMap)._writeToPathInternal(paths, index + 1, value);
  }

  void _deleteFromPath(String path) {
    final paths = path.split("/");
    if (paths.isEmpty) {
      return;
    }
    _deleteFromPathInternal(paths, 0);
  }

  void _deleteFromPathInternal(List<String> paths, int index) {
    if (paths.length - 1 <= index) {
      remove(paths[index]);
      return;
    }
    final p = paths[index];
    if (p.isEmpty) {
      return;
    }
    if (!containsKey(p) || this[p] == null || this[p] is! DynamicMap) {
      return;
    }
    (this[p] as DynamicMap)._deleteFromPathInternal(paths, index + 1);
  }
}
